//
//  AppDelegate.m
//  LuaHashMapTest
//
//  Created by Eric Wing on 3/7/13.
//  Copyright (c) 2013 PlayControl Software, LLC. All rights reserved.
//

#import "AppDelegate.h"

#import "ViewController.h"
#include "LuaHashMap.h"

@interface AppDelegate ()
@property(nonatomic, assign) LuaHashMap* luaHashMap;

@end

@implementation AppDelegate

- (void)dealloc
{
	LuaHashMap_Free(_luaHashMap);
	[_window release];
	[_viewController release];
    [super dealloc];
}

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    self.window = [[[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]] autorelease];
    // Override point for customization after application launch.
	if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
	    self.viewController = [[[ViewController alloc] initWithNibName:@"ViewController_iPhone" bundle:nil] autorelease];
	} else {
	    self.viewController = [[[ViewController alloc] initWithNibName:@"ViewController_iPad" bundle:nil] autorelease];
	}
	self.window.rootViewController = self.viewController;
    [self.window makeKeyAndVisible];
	
	LuaHashMap* lua_hash_map = LuaHashMap_Create();
	_luaHashMap = lua_hash_map;
	
	for(LuaHashMapIterator hash_iterator = LuaHashMap_GetIteratorAtBegin(lua_hash_map),
		hash_iterator_end = LuaHashMap_GetIteratorAtEnd(lua_hash_map);
			! LuaHashMap_IteratorIsEqual(&hash_iterator, &hash_iterator_end); LuaHashMap_IteratorNext(&hash_iterator) )
		{
			fprintf(stderr, "Price of %s: %lf\n",
					LuaHashMap_GetKeyStringAtIterator(&hash_iterator),
					LuaHashMap_GetCachedValueNumberAtIterator(&hash_iterator));
		}
	
	
	LuaHashMap* hash_map = _luaHashMap;
	
	
	LuaHashMap_SetValueNumberForKeyString(hash_map, 3.99, "milk");
	LuaHashMap_SetValueStringForKeyString(hash_map, "$4.34 9/10 per gallon", "gas");
	LuaHashMap_SetValuePointerForKeyString(hash_map, (void*)299, "bread");
	
	
	LuaHashMapIterator shared_iterator = LuaHashMap_GetIteratorAtBegin(hash_map);
	// Assumes at least 1 entry in the collection
	do
	{
		int keytype = LuaHashMap_GetKeyTypeAtIterator(&shared_iterator);
		int valuetype = LuaHashMap_GetCachedValueTypeAtIterator(&shared_iterator);
		switch(keytype)
		{
			case LUA_TLIGHTUSERDATA:
				fprintf(stderr, "\tKeyPointer:%zd", LuaHashMap_GetKeyPointerAtIterator(&shared_iterator));
				break;
			case LUA_TNUMBER:
				fprintf(stderr, "\tKeyNumber:%lf", LuaHashMap_GetKeyNumberAtIterator(&shared_iterator));
				break;
			case LUA_TSTRING:
				fprintf(stderr, "\tKeyString:%s", LuaHashMap_GetKeyStringAtIterator(&shared_iterator));
				break;
		}
		switch(valuetype)
		{
			case LUA_TLIGHTUSERDATA:
				fprintf(stderr, "\tValuePointer:%zd", LuaHashMap_GetCachedValuePointerAtIterator(&shared_iterator));
				break;
			case LUA_TNUMBER:
				fprintf(stderr, "\tValueNumber:%lf", LuaHashMap_GetCachedValueNumberAtIterator(&shared_iterator));
				break;
			case LUA_TSTRING:
				fprintf(stderr, "\tValueString:%s", LuaHashMap_GetCachedValueStringAtIterator(&shared_iterator));
				break;
		}
		fprintf(stderr, "\n");
	} while(LuaHashMap_IteratorNext(&shared_iterator));
	
    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application
{
	// Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
	// Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
	// Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
	// If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
	// Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
	// Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application
{
	// Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
}

@end
